package vieon.reactlibrary.player.core;

import androidx.annotation.NonNull;
import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.WritableMap;
import com.google.android.exoplayer2.C;
import com.google.android.exoplayer2.ExoPlaybackException;
import com.google.android.exoplayer2.Format;
import com.google.android.exoplayer2.PlaybackParameters;
import com.google.android.exoplayer2.analytics.AnalyticsListener;
import com.google.android.exoplayer2.audio.AudioAttributes;
import com.google.android.exoplayer2.decoder.DecoderCounters;
import com.google.android.exoplayer2.metadata.Metadata;
import com.google.android.exoplayer2.source.MediaSourceEventListener;
import com.google.android.exoplayer2.source.TrackGroupArray;
import com.google.android.exoplayer2.trackselection.TrackSelectionArray;
import com.google.android.exoplayer2.upstream.DataSpec;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import java.io.IOException;
import java.lang.reflect.Type;
import java.util.List;
import java.util.Map;

public class ExoPlayerAnalyticsListenerImpl implements AnalyticsListener {

  private VideoEventEmitter eventEmitter;
  private Gson gson;
  private ReactExoplayerView exoplayerView;

  public ExoPlayerAnalyticsListenerImpl(@NonNull VideoEventEmitter eventEmitter,
      ReactExoplayerView exoplayerView) {
    this.eventEmitter = eventEmitter;
    this.gson = new Gson();
    this.exoplayerView = exoplayerView;
  }

  @Override
  public void onPlayerStateChanged(EventTime eventTime, boolean playWhenReady, int playbackState) {

  }

  @Override public void onTimelineChanged(EventTime eventTime, int reason) {

  }

  @Override public void onPositionDiscontinuity(EventTime eventTime, int reason) {

  }

  @Override public void onSeekStarted(EventTime eventTime) {

  }

  @Override public void onSeekProcessed(EventTime eventTime) {

  }

  @Override public void onPlaybackParametersChanged(EventTime eventTime,
      PlaybackParameters playbackParameters) {

  }

  @Override public void onRepeatModeChanged(EventTime eventTime, int repeatMode) {

  }

  @Override public void onShuffleModeChanged(EventTime eventTime, boolean shuffleModeEnabled) {

  }

  @Override public void onLoadingChanged(EventTime eventTime, boolean isLoading) {

  }

  @Override public void onPlayerError(EventTime eventTime, ExoPlaybackException error) {
    if (this.exoplayerView != null) this.exoplayerView.handleOnError(error);

    WritableMap argMap = constructDefaultEventMap(eventTime);
    argMap.putString("ERROR_TYPE", "playback");
    argMap.putDouble("TIMESTAMP", System.currentTimeMillis());

    switch (error.type) {
      case ExoPlaybackException.TYPE_RENDERER:
        argMap.putString("ERROR_MESSAGE", "renderer" + error.getRendererException().getMessage());
        break;
      case ExoPlaybackException.TYPE_UNEXPECTED:
        argMap.putString("ERROR_MESSAGE",
            "unexpected" + error.getUnexpectedException().getMessage());
        break;
    }

    eventEmitter.error(argMap);
  }

  @Override public void onTracksChanged(EventTime eventTime, TrackGroupArray trackGroups,
      TrackSelectionArray trackSelections) {

  }

  @Override public void onLoadStarted(EventTime eventTime,
      MediaSourceEventListener.LoadEventInfo loadEventInfo,
      MediaSourceEventListener.MediaLoadData mediaLoadData) {

    WritableMap argMap = constructDefaultEventMap(eventTime);

    argMap.putDouble("TIMESTAMP", System.currentTimeMillis());

    argMap = this.constructLoadEventMap(loadEventInfo, mediaLoadData, argMap);

    eventEmitter.bufferStart(argMap);
  }

  @Override public void onLoadCompleted(EventTime eventTime,
      MediaSourceEventListener.LoadEventInfo loadEventInfo,
      MediaSourceEventListener.MediaLoadData mediaLoadData) {

    WritableMap argMap = constructDefaultEventMap(eventTime);

    argMap.putDouble("TIMESTAMP", System.currentTimeMillis());

    argMap = this.constructLoadEventMap(loadEventInfo, mediaLoadData, argMap);

    eventEmitter.bufferComplete(argMap);
  }

  @Override public void onLoadCanceled(EventTime eventTime,
      MediaSourceEventListener.LoadEventInfo loadEventInfo,
      MediaSourceEventListener.MediaLoadData mediaLoadData) {

    WritableMap argMap = constructDefaultEventMap(eventTime);

    argMap.putDouble("TIMESTAMP", System.currentTimeMillis());

    argMap = this.constructLoadEventMap(loadEventInfo, mediaLoadData, argMap);

    eventEmitter.bufferCancel(argMap);
  }

  @Override
  public void onLoadError(EventTime eventTime, MediaSourceEventListener.LoadEventInfo loadEventInfo,
      MediaSourceEventListener.MediaLoadData mediaLoadData, IOException error,
      boolean wasCanceled) {
    if (wasCanceled) {
      WritableMap argMap = constructDefaultEventMap(eventTime);

      argMap.putString("ERROR_TYPE", "source");

      argMap.putString("ERROR_MESSAGE", error.getMessage());

      argMap.putDouble("TIMESTAMP", System.currentTimeMillis());

      argMap = this.constructLoadEventMap(loadEventInfo, mediaLoadData, argMap);

      eventEmitter.error(argMap);
    }
  }

  @Override public void onDownstreamFormatChanged(EventTime eventTime,
      MediaSourceEventListener.MediaLoadData mediaLoadData) {
    if(mediaLoadData != null && mediaLoadData.trackFormat != null){
      eventEmitter.resolutionReport(mediaLoadData.trackFormat.height);
    }
  }

  @Override public void onUpstreamDiscarded(EventTime eventTime,
      MediaSourceEventListener.MediaLoadData mediaLoadData) {

  }

  @Override public void onMediaPeriodCreated(EventTime eventTime) {

  }

  @Override public void onMediaPeriodReleased(EventTime eventTime) {

  }

  @Override public void onReadingStarted(EventTime eventTime) {

  }

  @Override
  public void onBandwidthEstimate(EventTime eventTime, int totalLoadTimeMs, long totalBytesLoaded,
      long bitrateEstimate) {

  }

  @Override public void onSurfaceSizeChanged(EventTime eventTime, int width, int height) {

  }

  @Override public void onMetadata(EventTime eventTime, Metadata metadata) {

  }

  @Override public void onDecoderEnabled(EventTime eventTime, int trackType,
      DecoderCounters decoderCounters) {

  }

  @Override public void onDecoderInitialized(EventTime eventTime, int trackType, String decoderName,
      long initializationDurationMs) {

  }

  @Override
  public void onDecoderInputFormatChanged(EventTime eventTime, int trackType, Format format) {

  }

  @Override public void onDecoderDisabled(EventTime eventTime, int trackType,
      DecoderCounters decoderCounters) {

  }

  @Override public void onAudioSessionId(EventTime eventTime, int audioSessionId) {

  }

  @Override
  public void onAudioAttributesChanged(EventTime eventTime, AudioAttributes audioAttributes) {

  }

  @Override public void onVolumeChanged(EventTime eventTime, float volume) {

  }

  @Override public void onAudioUnderrun(EventTime eventTime, int bufferSize, long bufferSizeMs,
      long elapsedSinceLastFeedMs) {

  }

  @Override
  public void onDroppedVideoFrames(EventTime eventTime, int droppedFrames, long elapsedMs) {

  }

  @Override public void onVideoSizeChanged(EventTime eventTime, int width, int height,
      int unappliedRotationDegrees, float pixelWidthHeightRatio) {
  }

  private WritableMap constructDefaultEventMap(EventTime eventTime) {
    WritableMap argMap = Arguments.createMap();
    argMap.putDouble("CURRENT_PLAYBACK_POSITION", eventTime.currentPlaybackPositionMs);
    argMap.putDouble("TOTAL_BUFFERED_DURATION", eventTime.totalBufferedDurationMs);
    return argMap;
  }

  private WritableMap constructLoadEventMap(MediaSourceEventListener.LoadEventInfo loadEventInfo,
      MediaSourceEventListener.MediaLoadData mediaLoadData, WritableMap argMap) {
    if (loadEventInfo != null) {
      WritableMap loadEventMap = Arguments.createMap();

      loadEventInfo.responseHeaders.remove("access-control-allow-origin");
      loadEventInfo.responseHeaders.remove("access-control-allow-methods");
      loadEventInfo.responseHeaders.remove("access-control-allow-headers");
      loadEventInfo.responseHeaders.remove("access-control-allow-credentials");
      loadEventInfo.responseHeaders.remove("accept-ranges");

      Type responseHeaderType = new TypeToken<Map<String, List<String>>>() {
      }.getType();
      String responseHeaderJson = gson.toJson(loadEventInfo.responseHeaders, responseHeaderType)
          .replace("[", "")
          .replace("]", "");

      Type dataSpecType = new TypeToken<DataSpec>() {
      }.getType();
      String dataSpecJson = gson.toJson(loadEventInfo.dataSpec, dataSpecType);

      if (loadEventInfo.bytesLoaded >= 0) {
        loadEventMap.putDouble("LOADED_BYTES", loadEventInfo.bytesLoaded);
      }
      if (loadEventInfo.loadDurationMs >= 0) {
        loadEventMap.putDouble("LOAD_DURATION", loadEventInfo.loadDurationMs);
      }
      loadEventMap.putString("REQUEST_URI", loadEventInfo.uri.toString());
      loadEventMap.putString("RESPONSE_HEADER_JSON", responseHeaderJson);
      loadEventMap.putString("DATA_SPEC_JSON", dataSpecJson);

      argMap.putMap("LOAD_DATA", loadEventMap);
    }

    if (mediaLoadData != null) {
      WritableMap mediaDataMap = Arguments.createMap();

      Type trackFormatType = new TypeToken<Format>() {
      }.getType();
      String trackFormatJson = gson.toJson(mediaLoadData.trackFormat, trackFormatType);

      if (mediaLoadData.mediaStartTimeMs >= 0) {
        mediaDataMap.putDouble("MEDIA_START", mediaLoadData.mediaStartTimeMs);
      }
      if (mediaLoadData.mediaEndTimeMs >= 0) {
        mediaDataMap.putDouble("MEDIA_END", mediaLoadData.mediaEndTimeMs);
      }
      mediaDataMap.putString("DATA_TYPE", getDataType(mediaLoadData.dataType));
      mediaDataMap.putString("TRACK_TYPE", getTrackType(mediaLoadData.trackType));
      mediaDataMap.putString("TRACK_FORMAT_JSON", trackFormatJson);
      mediaDataMap.putString("TRACK_SELECTION_REASON",
          getSelectionReason(mediaLoadData.trackSelectionReason));

      argMap.putMap("MEDIA_DATA", mediaDataMap);
    }

    return argMap;
  }

  private String getSelectionReason(int reason) {
    switch (reason) {
      case C.SELECTION_REASON_UNKNOWN:
        return "UNKNOWN";
      /**
       * A selection reason constant for an initial track selection.
       */
      case C.SELECTION_REASON_INITIAL:
        return "INITIAL";
      /**
       * A selection reason constant for an manual (i.e. user initiated) track selection.
       */
      case C.SELECTION_REASON_MANUAL:
        return "MANUAL";
      /**
       * A selection reason constant for an adaptive track selection.
       */
      case C.SELECTION_REASON_ADAPTIVE:
        return "ADAPTIVE";
      /**
       * A selection reason constant for a trick play track selection.
       */
      case C.SELECTION_REASON_TRICK_PLAY:
        return "TRICK_PLAY";
      /**
       * Applications or extensions may define custom {@code SELECTION_REASON_*} constants greater than
       * or equal to this value.
       */
      case C.SELECTION_REASON_CUSTOM_BASE:
        return "CUSTOM_BASE";
      default:
        return "UNKNOWN";
    }
  }

  private String getTrackType(int trackType) {
    switch (trackType) {
      case C.TRACK_TYPE_UNKNOWN:
        /** A type constant for tracks of some default type, where the type itself is unknown. */
        return "UNKNOWN";
      case C.TRACK_TYPE_DEFAULT:
        return "DEFAULT";
      /** A type constant for audio tracks. */
      case C.TRACK_TYPE_AUDIO:
        return "AUDIO";
      /** A type constant for video tracks. */
      case C.TRACK_TYPE_VIDEO:
        return "VIDEO";
      /** A type constant for text tracks. */
      case C.TRACK_TYPE_TEXT:
        return "TEXT";
      /** A type constant for metadata tracks. */
      case C.TRACK_TYPE_METADATA:
        return "METADATA";
      /** A type constant for camera motion tracks. */
      case C.TRACK_TYPE_CAMERA_MOTION:
        return "CAMERA_MOTION";
      /** A type constant for a dummy or empty track. */
      case C.TRACK_TYPE_NONE:
        return "NONE";
      /**
       * Applications or extensions may define custom {@code TRACK_TYPE_*} constants greater than or
       * equal to this value.
       */
      case C.TRACK_TYPE_CUSTOM_BASE:
        return "CUSTOM_BASE";
      default:
        return "UNKNOWN";
    }
  }

  private String getDataType(int dataType) {
    switch (dataType) {
      case C.DATA_TYPE_UNKNOWN:
        /** A data type constant for media, typically containing media samples. */
        return "UNKNOWN";
      case C.DATA_TYPE_MEDIA:
        /** A data type constant for media, typically containing only initialization data. */
        return "MEDIA";
      case C.DATA_TYPE_MEDIA_INITIALIZATION:
        return "MEDIA_INITIALIZATION";
      case C.DATA_TYPE_DRM:
        return "DRM";
      case C.DATA_TYPE_MANIFEST:
        /** A data type constant for time synchronization data. */
        return "MANIFEST";
      case C.DATA_TYPE_TIME_SYNCHRONIZATION:
        /** A data type constant for ads loader data. */
        return "TIME_SYNCHRONIZATION";
      case C.DATA_TYPE_AD:
        /**
         * A data type constant for live progressive media streams, typically containing media samples.
         */
        return "AD";
      case C.DATA_TYPE_MEDIA_PROGRESSIVE_LIVE:
        /**
         * Applications or extensions may define custom {@code DATA_TYPE_*} constants greater than or
         * equal to this value.
         */
        return "MEDIA_PROGRESSIVE_LIVE";
      case C.DATA_TYPE_CUSTOM_BASE:
        return "CUSTOM_BASE";
      default:
        return "UNKNOWN";
    }
  }
}
