package vieon.reactlibrary.player.plugins.filmstrip;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Rect;
import android.net.Uri;
import android.os.Handler;
import android.os.Looper;
import android.text.TextUtils;
import android.util.Log;
import com.bumptech.glide.Glide;
import com.bumptech.glide.load.DataSource;
import com.bumptech.glide.load.engine.GlideException;
import com.bumptech.glide.request.RequestListener;
import com.bumptech.glide.request.target.Target;
import com.facebook.react.bridge.ReactContext;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.uimanager.ThemedReactContext;
import com.facebook.react.uimanager.ViewGroupManager;
import com.facebook.react.uimanager.annotations.ReactProp;
import com.google.android.exoplayer2.upstream.DefaultBandwidthMeter;
import com.google.android.exoplayer2.upstream.HttpDataSource;
import javax.annotation.Nullable;
import vieon.reactlibrary.player.core.DataSourceUtil;

public class ReactThumbnailViewManager extends ViewGroupManager<ThumbnailViewDisplay> {

  private static final String REACT_CLASS = "RNVieONFilmStrip";

  private static final String PROP_THUMB_SEEK = "thumbSeek";
  private static final String PROP_THUMB_SEEK_IMAGE = "image";
  private static final String PROP_THUMB_SEEK_VTT = "vtt";
  private static final String PROP_PROGRESS = "progress";


  private ReactContext reactContext;

  @Override
  public String getName() {
    return REACT_CLASS;
  }

  @Override
  protected ThumbnailViewDisplay createViewInstance(ThemedReactContext themedReactContext) {
    this.reactContext = themedReactContext;
    return new ThumbnailViewDisplay(themedReactContext);
  }


  @ReactProp(name = PROP_PROGRESS)
  public void setThumbProgress(final ThumbnailViewDisplay thumbnailViewComponent, double progress){
     thumbnailViewComponent.show((long)progress);
  }


  @ReactProp(name = PROP_THUMB_SEEK)
  public void setThumbSeek(final ThumbnailViewDisplay thumbnailViewDisplay, @Nullable ReadableMap src){
    Context context = thumbnailViewDisplay.getContext().getApplicationContext();
    String imageUri = src.hasKey(PROP_THUMB_SEEK_IMAGE) ? src.getString(PROP_THUMB_SEEK_IMAGE) : null;
    String vttUri = src.hasKey(PROP_THUMB_SEEK_VTT) ? src.getString(PROP_THUMB_SEEK_VTT) : null;

    if (TextUtils.isEmpty(imageUri) || TextUtils.isEmpty(vttUri)) {
      return;
    }


    ThumbnailDataTrack thumbnailDataTrack = new ThumbnailDataTrack();
    thumbnailDataTrack.setUrl(vttUri);
    thumbnailDataTrack.setImageUrl(imageUri);
    thumbnailDataTrack.setType(ThumbnailDataTrack.TYPE_WEBVTT_INDEX);

    HttpDataSource.Factory factory = DataSourceUtil.buildHttpDataSourceFactory(reactContext,new DefaultBandwidthMeter(),null);

    ThumbnailLoader loader =
        new DataSourceThumbnailLoader(factory.createDataSource());

    VttThumbnailProvider vttThumbnailProvider = new VttThumbnailProvider(reactContext,loader,thumbnailDataTrack.getImageUrl(),new Handler(
        Looper.getMainLooper()));
    vttThumbnailProvider.setSourceUri(Uri.parse(thumbnailDataTrack.getUrl()));

    ThumbnailViewComponent thumbnailViewComponent = new ThumbnailViewComponent(thumbnailViewDisplay.getContext());
    thumbnailViewComponent.setThumbnailProvider(vttThumbnailProvider);
    thumbnailViewDisplay.setThumbnailView(thumbnailViewComponent);
  }


}
