package vieon.reactlibrary.player.plugins.filmstrip;

//
// Source code recreated from a .class file by IntelliJ IDEA
// (powered by Fernflower decompiler)
//

import android.content.Context;
import android.graphics.Bitmap;
import android.net.Uri;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import com.facebook.react.bridge.ReactContext;
import com.google.android.exoplayer2.util.Util;
import java.io.File;
import java.util.Iterator;
import java.util.Set;
import java.util.TreeSet;
import java.util.concurrent.ExecutorService;
import vieon.reactlibrary.player.core.VideoEventEmitter;

abstract class AbstractThumbnailProvider implements ThumbnailProvider {
  private static final String TAG = "ThumbnailProvider";
  private final VideoEventEmitter eventEmitter;
  @Nullable
  protected Uri sourceUri;
  private final ExecutorService backgroundLoader;
  private final ExecutorService backgroundProvider;
  private final Object currentPositionLock = new Object();
  @NonNull
  protected final ImageDataCache imageDataCache;
  @NonNull
  protected final ThumbnailIndex index;

  private boolean initialized;
  protected ThumbnailLoader loader;
  private long lastRequestedPositionUs = -1L;
  private Callback lastRequestedCallback;
  private long currentRequestedPositionUs = -1L;
  private Handler handler;

  AbstractThumbnailProvider(ReactContext context,@NonNull final ThumbnailLoader loader, Handler handler) {
    this.loader = loader;
    this.backgroundLoader = Util.newSingleThreadExecutor("Thumbnail-Loader");
    this.backgroundProvider = Util.newSingleThreadExecutor("Thumbnail-Provider");
    this.imageDataCache = new ImageDataCache(context);
    this.index = new ThumbnailIndex();
    this.handler = handler;
    this.eventEmitter = new VideoEventEmitter(context);
  }


  protected abstract void loadIndex() throws Exception;

  public void setSourceUri(Uri sourceUri){
    this.sourceUri = sourceUri;
    AbstractThumbnailProvider.this.backgroundLoader.submit(new Runnable() {
      public void run() {
        AbstractThumbnailProvider.this.refreshThumbnails();
      }
    });
  }

  public void release(){
    synchronized(loader) {
      loader.dispose();
    }

    AbstractThumbnailProvider.this.backgroundLoader.shutdownNow();
    AbstractThumbnailProvider.this.backgroundProvider.shutdownNow();
  }

  @Nullable
  public DownloadableResource[] getDownloadableResources(String remoteManifestUrl, File basePath, Bundle bundle) {
    try {
      ThumbnailDataTrack data = (ThumbnailDataTrack)bundle.getParcelable("com.castlabs.thumbnail.data");
      Set<Uri> urls = new TreeSet();
      synchronized(this.index) {
        this.loadIndex();

        for(int i = 0; i < this.index.size(); ++i) {
          ThumbnailEntry entry = this.index.getByIndex(i);
          if (entry != null) {
            urls.add(entry.imageUri);
          }
        }
      }

      boolean isRelative = data != null && data.isRelativeUrl();
      DownloadableResource[] resources = new DownloadableResource[urls.size() + (this.sourceUri != null ? 1 : 0)];
      if (this.sourceUri != null) {
        resources[0] = new DownloadableResource();
        resources[0].uri = this.sourceUri;
        resources[0].length = -1L;
        resources[0].type = 4;
        resources[0].target = DownloadableResource.createTarget(this.sourceUri, basePath, remoteManifestUrl, isRelative).toString();
      }

      int i = this.sourceUri != null ? 1 : 0;

      DownloadableResource resource;
      for(Iterator var9 = urls.iterator(); var9.hasNext(); resources[i++] = resource) {
        Uri url = (Uri)var9.next();
        resource = new DownloadableResource();
        resource.length = -1L;
        resource.type = 4;
        resource.uri = url;
        resource.target = DownloadableResource.createTarget(url, basePath, remoteManifestUrl, isRelative).toString();
      }

      return resources;
    } catch (Exception var13) {
      Log.e("ThumbnailProvider", "Error while loading index to generate downloadable resources");
      return null;
    }
  }

  public void getThumbnail(final long positionUs, final Callback callback) {
    if (!this.initialized) {
      this.initialized = true;
      this.backgroundLoader.submit(new Runnable() {
        public void run() {
          AbstractThumbnailProvider.this.refreshThumbnails();
          AbstractThumbnailProvider.this.maybeRetrigger(-1L);
        }
      });
    }

    if (!this.index.contains(positionUs, true)) {
      this.lastRequestedCallback = callback;
      this.lastRequestedPositionUs = positionUs;
       this.eventEmitter.thumbLoadError("index not found");
    } else {
      synchronized(this.currentPositionLock) {
        this.currentRequestedPositionUs = positionUs;
      }

      this.backgroundProvider.submit(new Runnable() {
        public void run() {
          ThumbnailEntry entry = AbstractThumbnailProvider.this.index.get(positionUs, true);
          if (entry != null) {
            try {
              AbstractThumbnailProvider.this.imageDataCache.get(entry, new ImageDataCache.Callback() {
                @Override public void onBitmapRetrieve(@Nullable Bitmap bitmap) {
                  AbstractThumbnailProvider.this.maybeTriggerCallback(bitmap, positionUs, callback);
                }
              });
            } catch (Exception var3) {
              Log.e("ThumbnailProvider", "Error while loading thumbnail data: " + var3.getMessage(), var3);
            }

          }
        }
      });
    }
  }

  private void refreshThumbnails() {
    try {
      synchronized(this.index) {
        ThumbnailIndex oldIndex = this.index.copy();
        this.loadIndex();
        int numNewEntries = this.index.size();
        ThumbnailEntry firstNewEntry = numNewEntries > 0 ? this.index.getByIndex(0) : null;

        int i;
        for(i = 0; i < oldIndex.size(); ++i) {
          ThumbnailEntry oldEntry = oldIndex.getByIndex(i);
          if (firstNewEntry != null && oldEntry.imageUri.equals(firstNewEntry.imageUri)) {
            break;
          }

          this.imageDataCache.delete(oldEntry);
        }

        for(i = 0; i < this.index.size(); ++i) {
          if (Thread.interrupted()) {
            return;
          }

          this.imageDataCache.load(this.index.getByIndex(i), this.loader);
        }
      }
    } catch (Exception var9) {
      var9.printStackTrace();
      eventEmitter.thumbLoadError(var9.getMessage());
      //if (this.playerController != null) {
      //  this.playerController.getPlayerListeners().fireError(new CastlabsPlayerException(1, 24, "Error while downloading thumbnail data.", var9));
      //}
    }

  }

  private void maybeTriggerCallback(final Bitmap data, final long positionUs, final Callback callback) {
    boolean positionIsUpToDate;
    synchronized(this.currentPositionLock) {
      positionIsUpToDate = positionUs == this.currentRequestedPositionUs;
    }

    if (!Thread.interrupted() && positionIsUpToDate && data != null) {
      this.handler.post(new Runnable() {
        public void run() {
          callback.onThumbnailLoaded(positionUs, data);
        }
      });
    }

  }

  private void maybeRetrigger(long timestampUs) {
    if (this.lastRequestedCallback != null && this.lastRequestedPositionUs >= 0L) {
      if (timestampUs >= 0L && this.lastRequestedPositionUs > timestampUs) {
        return;
      }

      Callback callback = this.lastRequestedCallback;
      long pos = this.lastRequestedPositionUs;
      this.lastRequestedCallback = null;
      this.lastRequestedPositionUs = -1L;
      this.getThumbnail(pos, callback);
    }

  }
}

