package vieon.reactlibrary.player.plugins.filmstrip;

//
// Source code recreated from a .class file by IntelliJ IDEA
// (powered by Fernflower decompiler)
//

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Bitmap.Config;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Paint.Style;
import android.graphics.PorterDuff.Mode;
import android.graphics.Rect;
import android.graphics.RectF;
import android.util.AttributeSet;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import androidx.annotation.Nullable;
import com.facebook.react.views.view.ReactViewGroup;

public class DefaultThumbnailView extends ViewGroup {
  private static final String TAG = "ThumbnailRenderer";
  private static final float MAX_ASPECT_RATIO_DEFORMATION_FRACTION = 0.01F;
  @Nullable
  private ThumbnailProvider thumbnailProvider;
  private Bitmap thumbsBitmap;
  private Canvas thumbsCanvas;
  @Nullable
  private Paint borderPaint;
  private final Rect renderRect = new Rect();
  private Bitmap currentThumbnail;
  private boolean isSmallScreen = false;
  private Rect scaleDestination = new Rect();
  private RectF borderRect = new RectF();
  private ThumbnailProvider.Callback thumbnailCallback;
  private DefaultThumbnailView.Callback lastSizeCallback;
  private boolean inited = false;

  public DefaultThumbnailView(Context context) {
    super(context);
    this.thumbnailCallback = new DefaultThumbnailProviderCallback();
    this.init(context);

  }

  @Override protected void onLayout(boolean changed, int l, int t, int r, int b) {

  }

  protected void init(Context context) {
    this.setBackgroundResource(android.R.color.transparent);
    this.setFocusable(false);
    this.setFocusableInTouchMode(false);
    this.setWillNotDraw(false);
    this.isSmallScreen = !isTablet(context);
    //this.borderPaint = new Paint();
    //this.borderPaint.setColor(-16777216);
    //this.borderPaint.setStrokeWidth(10.0F);
    //this.borderPaint.setStyle(Style.STROKE);
  }

  public void setBorderPaint(@Nullable Paint borderPaint) {
    this.borderPaint = borderPaint;
  }

  public void setThumbnailProvider(@Nullable ThumbnailProvider thumbnailProvider) {
    this.thumbnailProvider = thumbnailProvider;
  }

  protected void show(long positionUs, DefaultThumbnailView.Callback sizeCallback) {
    if (this.thumbnailProvider == null) {
      Log.w("ThumbnailRenderer", "show() called but no ThumbnailProvider is set.");
    } else {
      this.lastSizeCallback = sizeCallback;
      if (positionUs < 0L) {
        this.hide();
      } else {
        this.setVisibility(VISIBLE);
      }

      this.thumbnailProvider.getThumbnail(positionUs, this.thumbnailCallback);
    }

  }

  public void hide() {
    this.lastSizeCallback = null;
    this.requestLayout();
  }

  protected void onDraw(Canvas canvas) {
    super.onDraw(canvas);
    this.updateBufferBitmap(this.getWidth(), this.getHeight());
    this.thumbsCanvas.drawColor(0, Mode.CLEAR);
    if (this.currentThumbnail != null && this.lastSizeCallback != null) {
      int thumbWidth = this.currentThumbnail.getWidth();
      int thumbHeight = this.currentThumbnail.getHeight();
      this.lastSizeCallback.getThumbnailRect(this.renderRect, thumbWidth, thumbHeight, this.isSmallScreen);
      int targetWidth = this.renderRect.width();
      int targetHeight = this.renderRect.height();
      float videoAspectRatio = (float)thumbWidth / (float)thumbHeight;
      if (videoAspectRatio > 0.0F) {
        //float viewAspectRatio = (float)targetWidth / (float)targetHeight;
        //float aspectDeformation = videoAspectRatio / viewAspectRatio - 1.0F;
        //if (Math.abs(aspectDeformation) > 0.01F) {
        //  if (aspectDeformation > 0.0F) {
        //    targetHeight = (int)((float)targetWidth / videoAspectRatio);
        //  } else {
        //    targetWidth = (int)((float)targetHeight * videoAspectRatio);
        //  }
        //}

        this.scaleDestination.set(this.renderRect.left, this.renderRect.top, this.renderRect.left + targetWidth, this.renderRect.top + targetHeight);
        this.thumbsCanvas.drawBitmap(this.currentThumbnail, (Rect)null, this.scaleDestination, (Paint)null);
        if (this.borderPaint != null) {
          this.borderRect.set((float)this.renderRect.left, (float)this.renderRect.top, (float)this.renderRect.right, (float)this.renderRect.bottom);
          this.thumbsCanvas.drawRoundRect(this.borderRect, 10.0F, 10.0F, this.borderPaint);
        }
      }
    }

    canvas.drawBitmap(this.thumbsBitmap, 0.0F, 0.0F, (Paint)null);
  }

  private void updateBufferBitmap(int w, int h) {
    if (this.thumbsBitmap == null || this.thumbsBitmap.getWidth() != w || this.thumbsBitmap.getHeight() != h) {
      if (this.thumbsBitmap != null) {
        this.thumbsBitmap.recycle();
      }

      this.thumbsBitmap = Bitmap.createBitmap(w,h, Config.ARGB_8888);
      this.thumbsCanvas = new Canvas(this.thumbsBitmap);
    }

  }

  private static boolean isTablet(Context context) {
    DisplayMetrics metrics = context.getResources().getDisplayMetrics();
    float yInches = (float)metrics.heightPixels / metrics.ydpi;
    float xInches = (float)metrics.widthPixels / metrics.xdpi;
    double diagonalInches = Math.sqrt((double)(xInches * xInches + yInches * yInches));
    return diagonalInches >= 6.5D;
  }

  public interface Callback {
    void getThumbnailRect(Rect var1, int var2, int var3, boolean var4);
  }

  class DefaultThumbnailProviderCallback implements ThumbnailProvider.Callback {
    DefaultThumbnailProviderCallback() {
    }

    public void onThumbnailLoaded(long positionUs, @Nullable Bitmap thumbnail) {
      DefaultThumbnailView.this.currentThumbnail = thumbnail;
      DefaultThumbnailView.this.invalidate();
    }
  }
}

