package vieon.reactlibrary.player.plugins.filmstrip;

//
// Source code recreated from a .class file by IntelliJ IDEA
// (powered by Fernflower decompiler)
//

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.util.Log;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import com.bumptech.glide.Glide;
import com.bumptech.glide.load.DataSource;
import com.bumptech.glide.load.engine.DiskCacheStrategy;
import com.bumptech.glide.load.engine.GlideException;
import com.bumptech.glide.request.RequestListener;
import com.bumptech.glide.request.target.Target;
import com.facebook.react.bridge.ReactContext;
import java.util.HashMap;
import java.util.Map;
import vieon.reactlibrary.player.core.VideoEventEmitter;

class ImageDataCache {
  private final Map<Uri, byte[]> imageDataCache = new HashMap();
  private final Object currentBitmapLock = new Object();
  private final ReactContext context;
  private Bitmap currentBitmap;
  private Uri currentBitmapUrl;

  public interface Callback {
    void onBitmapRetrieve(@Nullable Bitmap var3);
  }

  ImageDataCache(ReactContext context) {
    this.context =context;
  }

  boolean contains(@NonNull ThumbnailEntry entry) {
    if (entry == null) {
      return false;
    } else {
      synchronized(this.imageDataCache) {
        return this.imageDataCache.containsKey(entry.imageUri);
      }
    }
  }

  @Nullable void get(@NonNull ThumbnailEntry entry, Callback bitmapRetrieveCallback) throws Exception {
    if (entry == null) {
      bitmapRetrieveCallback.onBitmapRetrieve(null);
    } else {
      Glide.with(this.context).asBitmap().diskCacheStrategy(DiskCacheStrategy.ALL)
          .addListener(new RequestListener<Bitmap>() {
        @Override
        public boolean onLoadFailed(@Nullable GlideException e, Object model, Target<Bitmap> target,
            boolean isFirstResource) {
          bitmapRetrieveCallback.onBitmapRetrieve(null);
          return false;
        }

        @Override
        public boolean onResourceReady(Bitmap resource, Object model, Target<Bitmap> target,
            DataSource dataSource, boolean isFirstResource) {


          Bitmap result = resource;
          if (entry.gridSize != null) {
            if (entry.gridSize.x > 1 || entry.gridSize.y > 1) {
              int thumbWidth = result.getWidth() / entry.gridSize.x;
              int thumbHeight = result.getHeight() / entry.gridSize.y;
              int thumbX = entry.x * thumbWidth;
              int thumbY = entry.y * thumbHeight;
              result = Bitmap.createBitmap(result, thumbX, thumbY, thumbWidth, thumbHeight);
            }
          } else if (entry.x >= 0) {
            result = Bitmap.createBitmap(result, entry.x, entry.y, entry.w, entry.h);
          }

          bitmapRetrieveCallback.onBitmapRetrieve(result);
          return false;
        }
      }).load(entry.imageUri).preload();
    }
  }

  @Nullable
  public void load(@NonNull ThumbnailEntry entry, @NonNull ThumbnailLoader thumbnailLoader) throws
      Exception {
    byte[] data = null;
    boolean needToLoad = !this.contains(entry);
    if (needToLoad) {
      synchronized(thumbnailLoader) {
        data = thumbnailLoader.load(entry.imageUri);
      }

      if (data == null) {
        return;
      }

      synchronized(this.imageDataCache) {
        this.imageDataCache.put(entry.imageUri, data);
      }
    }

    entry.setDataLoaded(true);
  }

  public boolean delete(@NonNull ThumbnailEntry entry) {
    synchronized(this.imageDataCache) {
      return this.imageDataCache.remove(entry.imageUri) != null;
    }
  }
}

