package vieon.reactlibrary.player.plugins.filmstrip;

//
// Source code recreated from a .class file by IntelliJ IDEA
// (powered by Fernflower decompiler)
//

import android.content.Context;
import android.net.Uri;
import android.os.Handler;
import android.util.Log;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import com.facebook.react.bridge.ReactContext;
import com.google.android.exoplayer2.text.webvtt.WebvttParserUtil;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import vieon.reactlibrary.player.utils.IOUtils;

public class VttThumbnailProvider extends AbstractThumbnailProvider {

  private String imageUrl;

  public VttThumbnailProvider(ReactContext context,@NonNull ThumbnailLoader loader,String imageUrl, Handler handler) {
    super(context,loader,handler);
    this.imageUrl = imageUrl;
  }

  public void setImageUrl(String imageUrl){
    this.imageUrl = imageUrl;
  }

  protected void loadIndex() throws Exception {
    synchronized(this.loader) {
      byte[] bifData = this.loader.load(this.sourceUri);
      VttThumbnailProvider.VttParser vttParser = new VttThumbnailProvider.VttParser(this.index, this.sourceUri,this.imageUrl);
      vttParser.load(bifData);
    }
  }

  private static class VttParser {
    static final Pattern CUE_HEADER_PATTERN = Pattern.compile("^(\\S+)\\s+-->\\s+(\\S+)(.*)?$");
    static final Pattern GRID_COORDINATES_PATTERN = Pattern.compile("^xywh=(\\d+),(\\d+),(\\d+),(\\d+)$");
    private final ThumbnailIndex index;
    private final Uri sourceUri;
    private final String imageUrl;

    private VttParser(ThumbnailIndex index, Uri sourceUri, String imageUrl) {
      this.index = index;
      this.sourceUri = sourceUri;
      this.imageUrl = imageUrl;
    }

    void load(byte[] data) throws IOException {
      this.load((InputStream)(new ByteArrayInputStream(data)));
    }

    void load(InputStream inputStream) throws IOException {
      this.index.clear();
      BufferedReader bin = null;

      try {
        bin = new BufferedReader(new InputStreamReader(inputStream));
        this.readVttHeader(bin);
        ThumbnailEntry currentEntry = null;

        String line;
        while((line = bin.readLine()) != null) {
          if (!line.trim().isEmpty()) {
            if (currentEntry != null) {
              currentEntry = this.readVttCueLine(line, currentEntry);
              if (currentEntry != null) {
                this.index.add(currentEntry);
              }

              currentEntry = null;
            } else {
              currentEntry = this.readVttTimeLine(line);
            }
          }
        }
      } finally {
        IOUtils.closeQuietly(bin);
      }

    }

    @Nullable
    private ThumbnailEntry readVttTimeLine(String line) {
      Matcher cueHeaderMatcher = CUE_HEADER_PATTERN.matcher(line);
      if (cueHeaderMatcher.matches()) {
        try {
          long startTimeUs = WebvttParserUtil.parseTimestampUs(cueHeaderMatcher.group(1));
          ThumbnailEntry thumbnailEntry = new ThumbnailEntry();
          thumbnailEntry.timestampUs = startTimeUs/1000;
          return thumbnailEntry;
        } catch (NumberFormatException var6) {
          Log.w("MediaCodecInfo", "Skipping cue with bad header: " + cueHeaderMatcher.group());
        }
      }

      return null;
    }

    @Nullable
    private ThumbnailEntry readVttCueLine(String line, ThumbnailEntry currentEntry) {
      String id = line.trim();
      if (id.indexOf("#") > 0) {
        String[] split = id.split("#");
        if (split.length != 2) {
          Log.w("MediaCodecInfo", "Multiple anchors found, skipping " + line);
          return null;
        }

        id = split[0];
        Matcher gridMatcher = GRID_COORDINATES_PATTERN.matcher(split[1]);
        if (gridMatcher.matches()) {
          try {
            currentEntry.x = Integer.parseInt(gridMatcher.group(1));
            currentEntry.y = Integer.parseInt(gridMatcher.group(2));
            currentEntry.w = Integer.parseInt(gridMatcher.group(3));
            currentEntry.h = Integer.parseInt(gridMatcher.group(4));
          } catch (NumberFormatException var7) {
            Log.w("MediaCodecInfo", "Unable to parse grid coordinates, skipping " + line);
            return null;
          }
        }
      }

      currentEntry.imageUri = this.getEntryUri(imageUrl);
      return currentEntry;
    }

    @NonNull
    private Uri getEntryUri(String url) {
      Uri uri = null;
      if (!url.startsWith("http://") && !url.startsWith("https://")) {
        String vttUrl = this.sourceUri.toString();
        String vttBaseUrl = vttUrl.substring(0, vttUrl.lastIndexOf(47));
        uri = Uri.parse(vttBaseUrl).buildUpon().appendEncodedPath(url).build();
      } else {
        uri = Uri.parse(url);
      }

      return uri;
    }

    private void readVttHeader(BufferedReader bin) throws IOException {
      String firstLine = bin.readLine();
      if (firstLine == null || !firstLine.equals("WEBVTT")) {
        throw new IOException("Expected WEBVTT header. Got " + firstLine);
      }
    }
  }
}

