package vieon.reactlibrary.player.utils;

//
// Source code recreated from a .class file by IntelliJ IDEA
// (powered by Fernflower decompiler)
//


import android.net.Uri;
import androidx.annotation.NonNull;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class FileHelper {
  private static final String ENCODING_PLACEHOLDER_CHAR = "_";
  private static final String CLEAN_FILENAME_REGEXP = "[^a-zA-Z0-9-_\\.]";
  private static final String ENCODED_APID_PREFIX = "apid-";

  public FileHelper() {
  }

  public static String encodeFileName(String fileName) {
    return fileName.replaceAll("[^a-zA-Z0-9-_\\.]", "_");
  }

  public static List<File> listFiles(File rootDir, FileFilter filter, boolean recursive) {
    if (rootDir != null && rootDir.exists() && rootDir.isDirectory()) {
      File[] files = rootDir.listFiles();
      if (files == null) {
        return Collections.emptyList();
      } else {
        List<File> result = new ArrayList();
        File[] var5 = files;
        int var6 = files.length;

        for(int var7 = 0; var7 < var6; ++var7) {
          File file = var5[var7];
          if (filter == null || filter.accept(file)) {
            result.add(file);
          }

          if (recursive && file.isDirectory()) {
            result.addAll(listFiles(file, filter, true));
          }
        }

        return result;
      }
    } else {
      return Collections.emptyList();
    }
  }

  public static void copy(File sourceLocation, File targetLocation) throws IOException {
    if (sourceLocation != null && targetLocation != null) {
      if (!sourceLocation.exists()) {
        throw new IllegalArgumentException("Source " + sourceLocation + " does not exist");
      } else if (sourceLocation.isDirectory() && targetLocation.isFile()) {
        throw new IllegalArgumentException("You can not copy a directory to an existing file");
      } else {
        if (sourceLocation.isDirectory()) {
          if (!targetLocation.exists() && !targetLocation.mkdirs()) {
            throw new IOException("Cannot create dir " + targetLocation.getAbsolutePath());
          }

          File[] files = sourceLocation.listFiles();
          if (files == null) {
            return;
          }

          File[] var3 = files;
          int var4 = files.length;

          for(int var5 = 0; var5 < var4; ++var5) {
            File file = var3[var5];
            copy(file, new File(targetLocation, file.getName()));
          }
        } else {
          copyFile(sourceLocation, targetLocation);
        }

      }
    } else {
      throw new NullPointerException("NULL source or target are not permitted");
    }
  }

  private static void copyFile(File sourceFile, File targetFile) throws IOException {
    if (sourceFile != null && targetFile != null) {
      if (!sourceFile.isDirectory() && !targetFile.isDirectory()) {
        File directory = targetFile.getParentFile();
        if (directory != null && (directory.exists() || directory.mkdirs())) {
          InputStream in = new FileInputStream(sourceFile);
          OutputStream out = new FileOutputStream(targetFile);
          IOUtils.copy(in, out);
          IOUtils.closeQuietly(in);
          IOUtils.closeQuietly(out);
        } else {
          throw new IOException("Cannot create parent dir for " + targetFile.getAbsolutePath());
        }
      } else {
        throw new IllegalArgumentException("Neither source or target can be directories");
      }
    } else {
      throw new NullPointerException("NULL source or target not permitted");
    }
  }

  public static String encodeApid(String apid) {
    String encodedApid = "apid-";
    if (apid != null && !apid.equalsIgnoreCase(".") && !apid.equalsIgnoreCase("..")) {
      try {
        encodedApid = encodedApid + URLEncoder.encode(apid, "UTF-8");
        return encodedApid;
      } catch (UnsupportedEncodingException var3) {
        throw new RuntimeException("UTF-8 is an unknown encoding");
      }
    } else {
      return encodedApid;
    }
  }

  public static String decodeApid(String encodedApi) {
    String apid = encodedApi.substring("apid-".length());

    try {
      apid = URLDecoder.decode(apid, "UTF-8");
      return apid;
    } catch (UnsupportedEncodingException var3) {
      throw new RuntimeException("UTF-8 is an unknown encoding");
    }
  }

  public static void deleteRecursive(File fileOrDirectory) {
    if (fileOrDirectory.isDirectory()) {
      File[] files = fileOrDirectory.listFiles();
      if (files == null) {
        return;
      }

      File[] var2 = files;
      int var3 = files.length;

      for(int var4 = 0; var4 < var3; ++var4) {
        File child = var2[var4];
        deleteRecursive(child);
      }
    }

    fileOrDirectory.delete();
  }

  public static List<String> readLines(File file) throws IOException {
    BufferedReader reader = null;

    try {
      reader = new BufferedReader(new FileReader(file));
      List<String> lines = new ArrayList();

      for(String line = reader.readLine(); line != null; line = reader.readLine()) {
        lines.add(line);
      }

      ArrayList var7 = (ArrayList) lines;
      return var7;
    } finally {
      IOUtils.closeQuietly(reader);
    }
  }

  @NonNull
  public static String convertToLocalFilePath(@NonNull Uri uri) {
    String host = uri.getHost();
    String path = uri.getPath();
    String query = uri.getQuery();
    String fragment = uri.getFragment();
    StringBuilder sb = new StringBuilder();
    if (host != null) {
      sb.append("/");
      sb.append(host);
    }

    if (path != null) {
      sb.append(path);
    }

    if (query != null) {
      sb.append("?");
      sb.append(query);
    }

    if (fragment != null) {
      sb.append("#");
      sb.append(fragment);
    }

    return sb.toString();
  }
}

